package Authen::SPNEGO::GSSAPI::SecurityContext;

use strict;
use Authen::SPNEGO::GSSAPI 0.03 qw(gssapi_token_2_read_handle GSS_C_NO_CONTEXT gss_import_name hr_errormessage gss_context_time);
use Carp;

our $VERSION = '0.02';



sub new  {
  my $pkg = shift;
  #-----------------------------------------
  # check input params
  #
  my ($p) = @_;
  $p || croak 'please input parameters to new()!';
  exists $p->{'PRINCIPAL'} || croak 'missing required param PRINCIPAL';
  $p->{'PRINCIPAL'} || croak 'required param PRINCIPAL is empty!';
  #-----------------------------------------
  #  object setup
  #
  my $self;
  my ( $sec_context, $output_token);
  my $nocontext = GSS_C_NO_CONTEXT;
  my ($majorstate, $minorstate, $gss_servername ) = gss_import_name( $p->{'PRINCIPAL'} );
  unless ( $majorstate ) {
    ($majorstate, $minorstate, $sec_context, $output_token ) = Authen::SPNEGO::GSSAPI::achim_gss_init_sec_context($gss_servername, $nocontext);
    unless ( $majorstate ) {
       $self->{'XSTOKEN'} = $output_token;
       $self->{'XSSECHANDLE'} = $sec_context;
       $self->{'PRINC'} = $p->{'PRINCIPAL'};
       bless $self, $pkg;
       return $self;
    }
  }

  if ( $majorstate && die_on_error() ) {
     die join "\n", hr_errormessage( $majorstate, $minorstate)
  }
  return undef;
}
#--------------------------------------------
sub actual_ttl_secs {
    my $self = shift;
    exists  $self->{'XSSECHANDLE'} || die 'missing XSSECHANDLE';
    my ($majorstate, $minorstate, $secs) =  gss_context_time( $self->{'XSSECHANDLE'} );
    unless ( $majorstate  ) {
       return $secs;
    } else {
       die join "\n", hr_errormessage( $majorstate, $minorstate)
    }
}
#--------------------------------------------
sub token_read_handle {
    my $self = shift;
    exists  $self->{'XSTOKEN'} || die 'missing XSTOKEN';
    return gssapi_token_2_read_handle( $self->{'XSTOKEN'} );
}
#--------------------------------------------
# dummy for possible changeable die behavior
sub die_on_error { return 1; };

1;
#--------------------------------------------

__END__


=head1 NAME

Authen::SPNEGO::GSSAPI::SecurityContex
- Perl extension for using GSSAPI (higher level Inteface)

=head1 SYNOPSIS


use ExtUtils::testlib;
use MIME::Base64;

use Authen::SPNEGO::GSSAPI::SecurityContext;

eval {
   my $sec_context = Authen::SPNEGO::GSSAPI::SecurityContext->new(
                   { PRINCIPAL => 'HTTP/moerbsen.grolmsnet.lan' } );
   print "\n Time To live (TLL) of context: ",
          $sec_context->actual_ttl_secs();
   print "\n\n";
   *HAND = $sec_context->token_read_handle();

   my $temp = $/;
   local($/) = undef;  # slurp
   print encode_base64(<HAND>);
   close HAND;
};
if ($@) {
   print "\n Oh my good, runtimeerror!!!1111!\n\n", $@;
   exit;
}

=head1 METHODS

=head2 Constructor

$seccontext = Authen::SPNEGO::GSSAPI::SecurityContext->new( { ...params... );

=head3 params of new(  )

=over

=item PRINCIPAL (required)

The serviceprincipalname of server you want to connect to
example: HTTP/moerbsen.grolmsnet.lan@GROLMSNET.LAN
ist a webserver host moerbsen.grolmsnet.lan in Kerberos-Realm GROLMSNET.LAN

=back

=head2 Object Methods

=over

=item actual_ttl_secs()

returns the time to live (from now) of security handle in seconds.

=item token_read_handle()

return a filehandle the actual (binary!)
GSSAPI-Token can be read from.
This token is to be sent to the server via the socket
(or base64 encoded in case of HTTP, see SPNEGO-Draft for details)

=back

=head1 SEE ALSO

perldoc Authen::SPNEGO::GSSAPI

=head2 Standards

RfC 1509 describes the GSSAPI C-Bindings wrapped here



=head1 AUTHOR

Achim Grolms, E<lt>achim@grolmsnet.deE<gt>

many thanks to Harald Joerg who helped and contributed as usual :-)

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Achim Grolms

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.4 or,
at your option, any later version of Perl 5 you may have available.


=cut